#!/usr/bin/python
# -*- coding: UTF-8 -*-

from cStringIO import StringIO
from flask import *
from flask_compress import Compress
from threading import Thread
from werkzeug import *
from werkzeug.utils import secure_filename

import hashlib
import logging
import os
import tempfile
import urllib
import zipfile

from MDmisc.egit import git_version
from MDmisc.RecursiveDefaultDict import defDict

from functions import *

################################################################################

logging.getLogger().setLevel( logging.INFO )

################################################################################

api = Flask( __name__ )
Compress( api )

UPLOAD_FOLDER = os.path.join( tempfile.gettempdir(), "NIST" )
api.config[ 'UPLOAD_FOLDER' ] = UPLOAD_FOLDER

ALLOWED_EXTENSIONS = [
    '.nist', '.nst',
    '.an2', '.an2k',
    '.xml',
    '.lffs', '.lff',
    '.lfis', '.lfi',
    '.irq', '.irr', '.isr',
    '.srl',
    '.lsmq', '.lsmr',
    '.lpnq', '.lpn', '.lpnr',
    '.ulac', '.uld',
    '.ular', '.uldr', '.uuld',
    '.errl', '.erri', '.erra', '.err'
]

baseurl = os.environ.get( "BASEURL", "" )
envtype = os.environ.get( "ENVTYPE", "" )

################################################################################
#    Generic routing

@api.route( baseurl + '/ping' )
def ping():
    return "pong"

@api.route( baseurl + '/version' )
def version():
    try:
        return jsonify( git_version() )
    except:
        try:
            from version import __version__
            return jsonify( { 'error': False, 'data': __version__ } )
        except:
            return jsonify( { 'error': False, 'data': 'unknown' } )

################################################################################
#    Error handeling

@api.errorhandler( 404 )
def http_error_handler( error ):
    return redirect( baseurl )

################################################################################
#    Get all the routes

@api.route( baseurl + '/routes' )
def list_routes():
    output = []
    for rule in api.url_map.iter_rules():
        options = {}
        for arg in rule.arguments:
            options[ arg ] = "[{0}]".format( arg )

        methods = ','.join( rule.methods )
        url = url_for( rule.endpoint, **options )
        output.append( {
            'method': methods,
            'url': urllib.unquote( url )
        } )
    
    return jsonify( output )

################################################################################
#    Generic file serving

@api.route( baseurl + '/cdn/<directory>/<subpath>' )
def send_cdn( directory, subpath ):
    if not directory in [ 'js', 'css' ]:
        abort( 403 )
    elif subpath.endswith( ".map" ):
        return ""
    else:
        return send_from_directory( "./cdn/" + directory, subpath )

@api.route( baseurl + '/app/<path>' )
def send_app_files( path ):
    return send_from_directory( 'app', path )

################################################################################
#    File upload

@api.route( baseurl + '/upload', methods = [ 'GET', 'POST' ] )
def upload_file():
    if request.method == 'POST':
        if 'file' not in request.files:
            return jsonify( { 'error': True, 'message': 'No file in the POST request' } )
        
        file = request.files[ 'file' ]
        
        if file.filename == '':
            return jsonify( { 'error': True, 'message': 'No file selected' } )
        
        _, ext = os.path.splitext( file.filename )
        if file and ext.lower() in ALLOWED_EXTENSIONS:
            filename = secure_filename( file.filename.lower() )
            iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
            
            file.save( os.path.join( UPLOAD_FOLDER, filename ) )
            n = NISTf_auto( UPLOAD_FOLDER + "/" + filename )
            
            Thread( target = process, args = ( UPLOAD_FOLDER + "/" + filename, UPLOAD_FOLDER + "/" + iphash, filename ) ).start()
            
            return jsonify( { 'error': False, 'file': filename, 'hash': n.hash() } )
        
        else:
            return jsonify( { 'error': True, 'message': 'Format not allowed' } )

@api.route( baseurl + '/info/<h>/<filename>' )
def get_info( h, filename ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isfile( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename ):
        checktime( UPLOAD_FOLDER, iphash, filename )
        
        n = NISTf_auto( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename )
        
        ntypes = n.get_ntype()
        sizes = defDict()
        idcs = defDict()
        
        for ntype in ntypes:
            for idc in n.get_idc( ntype ):
                idcs[ ntype ][ idc ] = 1
                try:
                    fpc = n.get_field( ( ntype, fpcfield[ ntype ] ), idc )
                    sizes[ ntype ][ fpc ] = n.get_size( idc = idc )
                
                except:
                    continue
        
        return jsonify( {
            'error': False,
            'data': {
                'ntypes': ntypes,
                'sizes': sizes,
                'idcs': idcs
            }
        } )
    
    else:
        return jsonify( {
           'error': True,
           'message': 'File not found' 
        } ) 

@api.route( baseurl + '/dump/<h>/<filename>' )
def get_txt_dump( h, filename ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isfile( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename ):
        checktime( UPLOAD_FOLDER, iphash, filename )
        
        n = NISTf_auto( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename )
        
        data = n.dump( maxwidth = 160 )
        
        return jsonify( {
            'error': False,
            'data': data
        } )
    
    else:
        return jsonify( {
           'error': True,
           'message': 'File not found' 
        } )

@api.route( baseurl + '/<uuid>/dump' )
def dumpui( uuid ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid ):
        checktime( UPLOAD_FOLDER, iphash, uuid )
        
        lst = os.listdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid )
        name = uuid
        filename = None
        for f in lst:
            n, ext = os.path.splitext( f )
            if ext in ALLOWED_EXTENSIONS:
                name = n
                filename = f
        
        return render_template( 
            "dumpui.html",
            uuid = uuid,
            name = name,
            filename = filename,
            baseurl = baseurl,
            envtype = envtype
        )
    
    else:
        abort( 404 )

@api.route( baseurl + '/<uuid>/latent' )
def dumplatent( uuid ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid ):
        checktime( UPLOAD_FOLDER, iphash, uuid )
        
        lst = os.listdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid )
        name = uuid
        filename = None
        for f in lst:
            n, ext = os.path.splitext( f )
            if ext in ALLOWED_EXTENSIONS:
                name = n
                filename = f
        
        return render_template( 
            "latentui.html",
            uuid = uuid,
            name = name,
            filename = filename,
            baseurl = baseurl,
            envtype = envtype
        )
    else:
        abort( 404 )

@api.route( baseurl + '/img/<h>/<filename>' )
def get_file( h, filename ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isfile( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename ):
        return send_file( UPLOAD_FOLDER + "/" + iphash + "/" + h + "/" + filename, mimetype = 'image/png' )
    else:
        abort( 409 )

@api.route( baseurl + '/notavailable.png' )
def get_notavaiable_file():
    return send_file( "./app/not_available.png", mimetype = 'image/png' )

@api.route( baseurl + '/favicon.ico' )
def favicon():
    return ""

@api.route( baseurl + '/' )
def mainpage():
    return render_template( "dropzone.html", baseurl = baseurl, envtype = envtype )

@api.route( baseurl + '/<uuid>' )
def mainui( uuid ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid ):
        checktime( UPLOAD_FOLDER, iphash, uuid )
            
        lst = os.listdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid )
        name = uuid
        filename = None
        for f in lst:
            n, ext = os.path.splitext( f )
            if ext in ALLOWED_EXTENSIONS:
                name = n
                filename = f
        
        return render_template( 
            "ui.html",
            uuid = uuid,
            name = name,
            filename = filename,
            baseurl = baseurl,
            envtype = envtype
        )
    
    else:
        abort( 404 )

@api.route( baseurl + '/<uuid>.zip' )
def get_archive( uuid ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    
    if os.path.isdir( UPLOAD_FOLDER + "/" + iphash + "/" + uuid ):
        checktime( UPLOAD_FOLDER, iphash, uuid )
        
        io = StringIO()
        
        filename = uuid
        
        with zipfile.ZipFile( io, 'w', zipfile.ZIP_DEFLATED ) as fp:
            for root, dirs, files in os.walk( UPLOAD_FOLDER + "/" + iphash + "/" + uuid ):
                for file in files:
                    name, ext = os.path.splitext( file )
                    if ext in ALLOWED_EXTENSIONS:
                        filename = name
                    
                    if "_preview" in file:
                        continue
                    else:
                        fp.write( os.path.join( root, file ), file )
        
        io.seek( 0 )
        
        return send_file( io, attachment_filename = filename + ".zip", as_attachment = True )
    
    else:
        abort( 404 )

@api.route( baseurl + '/done/<uuid>' )
def is_done( uuid ):
    iphash = hashlib.md5( get_ip()[ 1 ] ).hexdigest()
    try:
        return jsonify( {
            'error': False,
            'isdone': os.path.isfile( UPLOAD_FOLDER + "/" + iphash + "/" + uuid + "/done.txt" )
        } )
    except:
        return jsonify( {
            'error': True
        } )

@api.route( baseurl + "/ip" )
def getip():
    c, ip = get_ip()
    return jsonify( {
        'type': c,
        'ip': ip
    } )

################################################################################
#    Main runner

if not os.path.isdir( UPLOAD_FOLDER ):
    os.makedirs( UPLOAD_FOLDER )

if __name__ == '__main__':
    api.run( debug = True, host = "0.0.0.0", threaded = True )
